/*   
 * Copyright 2007 Sun Microsystems, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package com.sun.syndication.propono.atom.server.impl;

import java.util.Iterator;
import org.jdom.Document;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;


import com.sun.syndication.feed.WireFeed;
import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;
import com.sun.syndication.feed.atom.Link;
import com.sun.syndication.io.FeedException;
import com.sun.syndication.io.WireFeedInput;
import com.sun.syndication.io.WireFeedOutput;
import com.sun.syndication.propono.atom.common.Collection;

import com.sun.syndication.propono.atom.server.AtomException;
import com.sun.syndication.propono.atom.server.AtomNotFoundException;
import com.sun.syndication.propono.atom.server.AtomServlet;
import com.sun.syndication.propono.utils.Utilities;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;
import javax.activation.FileTypeMap;
import javax.activation.MimetypesFileTypeMap;

/**
 * File based Atom collection implementation. This is the heart of the 
 * file-based Atom service implementation. It provides methods for adding, 
 * getting updating and deleting Atom entries and media entries.
 */
public class FileBasedCollection extends Collection {
    private String baseDir = null;
    private String handle = null;
    private String singular = null;
    private String collection = null;
    private String appuri = null;
    
    private static final String FEED_TYPE = "atom_1.0";

    /**
     * Construct by providing title (plain text, no HTML), a workspace handle,
     * a plural collection name (e.g. entries), a singular collection name
     * (e.g. entry), the base directory for file storage, the content-type
     * range accepted by the collection and the root Atom protocol URI for the
     * service.
     * @param title Title of collection (plain text, no HTML)
     * @param handle Workspace handle
     * @param collection Collection handle, plural
     * @param singular Collection handle, singular
     * @param baseDir Base directory for file storage
     * @param accept Content type range accepted by collection
     * @param appuri URI of Atom protocol service
     */
    public FileBasedCollection(
        String title,
        String handle, 
        String collection, 
        String singular, 
        String baseDir,
        String accept,
        String appuri) {       
        super(title, "text", appuri + "/" + handle + "/" + collection);
        
        this.handle = handle;
        this.collection = collection;
        this.singular = singular;
        this.baseDir = baseDir;
        this.appuri = appuri;
        
        setAccept(accept);
    }
    
    /**
     * Get feed document representing collection.
     * @throws com.sun.syndication.propono.atom.server.AtomException On error retrieving feed file.
     * @return Atom Feed representing collection.
     */
    public Feed getFeedDocument() throws AtomException {
        InputStream in = null;        
        synchronized(FileStore.getFileStore()) {
            in = FileStore.getFileStore().getFileInputStream(getFeedPath());
            
            if (in == null) {
                in = createDefaultFeedDocument(getHref());
            }
        }        
        try {
            WireFeedInput input = new WireFeedInput();
            WireFeed wireFeed = input.build(new InputStreamReader(in));
            return (Feed)wireFeed;
        } catch (FeedException ex) {
            throw new AtomException(ex);
        }
    }   
    
    /**
     * Add entry to collection.
     * @param entry Entry to be added to collection. Entry will be saved to disk in a 
     * directory under the collection's directory and the path will follow the 
     * pattern [collection-plural]/[entryid]/entry.xml. The entry will be added 
     * to the collection's feed in [collection-plural]/feed.xml.
     * @throws java.lang.Exception On error.
     * @return Entry as it exists on the server.
     */
    public Entry addEntry(Entry entry) throws Exception {
        Feed f = getFeedDocument();

        String fsid = FileStore.getFileStore().getNextId();            
        Entry storedEntry = updateAtomEntry(entry, fsid);

        // Update feed file
        updateFeedDocumentWithNewEntry(f, storedEntry);

        // Save entry to file
        String entryPath = getEntryPath(fsid);  
        OutputStream os = FileStore.getFileStore().getFileOutputStream(entryPath);
        AtomServlet.serializeEntry(storedEntry, os);
        os.flush();
        os.close();   
        
        return storedEntry;
    }
    
    /**
     * Add media entry to collection. Accepts a media file  to be added to collection. 
     * The file will be saved to disk in a directory under the collection's directory 
     * and the path will follow the pattern <code>[collection-plural]/[entryid]/media/[entryid]</code>.
     * An Atom entry will be created to store metadata for the entry and it will exist 
     * at the path <code>[collection-plural]/[entryid]/entry.xml</code>.
     * The entry will be added to the collection's feed in [collection-plural]/feed.xml.
     * @param title Title of new entry
     * @param slug String to be used in file-name
     * @param contentType Content type of data
     * @param is Source of media data
     * @throws java.lang.Exception On Error
     * @return New entry as it exists on the server
     */
    public Entry addMediaEntry(String title, String slug, String contentType, InputStream is) throws Exception {
        
        // Save media file temp file
        String fileName = createFileName((slug != null) ? slug : title, contentType);                 
        File tempFile = File.createTempFile(fileName, "tmp");
        FileOutputStream fos = new FileOutputStream(tempFile);
        Utilities.copyInputToOutput(is, fos);
        fos.close();

        // Save media file
        FileInputStream fis = new FileInputStream(tempFile);
        saveFile(fileName, contentType, tempFile.length(), fis);
        fis.close();
        File resourceFile = 
            new File(getEntryMediaPath(fileName));

        // Create media-link entry
        Entry atomEntry = updateAtomResourceEntry(new Entry(), resourceFile);

        // Update feed with new entry
        Feed f = getFeedDocument();
        updateFeedDocumentWithNewEntry(f, atomEntry);

        // Save media-link entry
        String entryPath = getEntryPath(fileName);  
        OutputStream os = FileStore.getFileStore().getFileOutputStream(entryPath);
        AtomServlet.serializeEntry(atomEntry, os);
        os.flush();
        os.close();
        
        return atomEntry;
    }
    
    /**
     * Get an entry from the collection.
     * @param fileName Internal ID of entry to be returned
     * @throws java.lang.Exception On error
     * @return Entry specified by fileName/ID
     */
    public Entry getEntry(String fileName) throws Exception {
        if (fileName.endsWith(".media-link")) {
            fileName = fileName.substring(0, fileName.length() - ".media-link".length());
        }

        String entryPath = getEntryPath(fileName); 

        checkExistence(entryPath);
        InputStream in = FileStore.getFileStore().getFileInputStream(entryPath);

        String filePath = getEntryMediaPath(fileName);                    
        File resource = new File(fileName);
        if (resource.exists()) {
            return loadAtomResourceEntry(in, resource);
        }
        return loadAtomEntry(in);        
    }
    
    /**
     * Update an entry in the collection.
     * @param entry Updated entry to be stored
     * @param fileName Internal ID of entry
     * @throws java.lang.Exception On error
     * @return Updated entry as it exists on server
     */
    public Entry updateEntry(Entry entry, String fileName) throws Exception {
        Feed f = getFeedDocument();

        if (fileName.endsWith(".media-link")) {
            fileName = fileName.substring(0, fileName.length() - ".media-link".length());
        }

        Entry updatedAtomEntry = updateAtomEntry(entry, fileName);
        updateFeedDocumentWithExistingEntry(f, updatedAtomEntry);
        String entryPath = getEntryPath(fileName);

        OutputStream os = FileStore.getFileStore().getFileOutputStream(entryPath);
        AtomServlet.serializeEntry(updatedAtomEntry, os);
        os.flush();
        os.close();
        
        return updatedAtomEntry;
    }
    
    /**
     * Update media associated with a media-link entry.
     * @param fileName Internal ID of entry being updated
     * @param contentType Content type of data
     * @param is Source of updated data
     * @throws java.lang.Exception On error
     * @return Updated Entry as it exists on server
     */
    public Entry updateMediaEntry(String fileName, String contentType, InputStream is) throws Exception {

        File tempFile = File.createTempFile(fileName, "tmp");
        FileOutputStream fos = new FileOutputStream(tempFile);
        Utilities.copyInputToOutput(is, fos);
        fos.close();

        // Update media file
        FileInputStream fis = new FileInputStream(tempFile);
        saveFile(fileName, contentType, tempFile.length(), fis);
        fis.close();
        File resourceFile = 
            new File(getEntryMediaPath(fileName));

        // Load media-link entry to return
        String entryPath = getEntryPath(fileName);
        InputStream in = FileStore.getFileStore().getFileInputStream(entryPath);
        Entry atomEntry = loadAtomResourceEntry(in, resourceFile);

        // Update feed with new entry
        Feed f = getFeedDocument();
        updateFeedDocumentWithExistingEntry(f, atomEntry);

        // Save updated media-link entry
        OutputStream os = FileStore.getFileStore().getFileOutputStream(entryPath);
        AtomServlet.serializeEntry(atomEntry, os);
        os.flush();
        os.close();  
        
        return atomEntry;
    }
    
    /**
     * Delete an entry and any associated media file.
     * @param fileName Internal ID of entry
     * @throws java.lang.Exception On error
     */
    public void deleteEntry(String fileName) throws Exception {
        if (fileName.endsWith(".media-link")) {
            fileName = fileName.substring(0, fileName.length() - ".media-link".length());
        }

        // Remove entry from Feed
        String entryId = appuri + "/" + handle + "/" + singular + "/" + fileName;
        Feed feed = getFeedDocument();
        updateFeedDocumentRemovingEntry(feed, entryId);

        String entryDir = getEntryDirPath(fileName);
        FileStore.getFileStore().deleteDirectory(entryDir);         
    }
    
    private String getBaseDir() {
        return baseDir;
    }
        
    private String getFeedPath() {
        return getBaseDir() + File.separator + handle  
            +  File.separator + collection + File.separator+ "feed.xml";
    }
        
    private String getEntryDirPath(String id) {
        return  getBaseDir() + File.separator  + handle 
            + File.separator + collection + File.separator + id;
    }
    
    private String getEntryPath(String id) {
        return getEntryDirPath(id) + File.separator + "entry.xml";
    }
    
    private String getEntryMediaPath(String id) {
        return getEntryDirPath(id) + File.separator + "media" + File.separator + id;
    }
    
    private static void checkExistence(String path) throws AtomNotFoundException{
        if (!FileStore.getFileStore().exists(path)) {
            throw new AtomNotFoundException("Entry does not exist");
        }
    }
            
    private void updateFeedDocumentWithNewEntry(Feed f, Entry e) throws AtomException {
        f.getEntries().add(e);
        updateFeedDocument(f);
    }
    
    private void updateFeedDocumentRemovingEntry(Feed f, String id) throws AtomException {
        Entry e = findEntry(id, f);
        
        f.getEntries().remove(e);
        
        updateFeedDocument(f);
    }
    
    private void updateFeedDocumentWithExistingEntry(Feed f, Entry e) throws AtomException {
        Entry old = findEntry(e.getId(), f);
        f.getEntries().remove(old);
        f.getEntries().add(0, e);
        
        updateFeedDocument(f);
    }
    
    private Entry findEntry(String id, Feed f) {
        List l = f.getEntries();
        for (Iterator it = l.iterator(); it.hasNext();) {
            Entry e = (Entry)it.next();
            if (id.equals(e.getId()))
                return e;
        }
        
        return null;
    }
    
    private void updateFeedDocument(Feed f) throws AtomException{
        try {
            synchronized(FileStore.getFileStore()) {                
                WireFeedOutput wireFeedOutput = new WireFeedOutput();
                Document feedDoc = wireFeedOutput.outputJDom( f );
                XMLOutputter outputter = new XMLOutputter();
                outputter.setFormat(Format.getPrettyFormat());
                outputter.output(feedDoc, 
                    FileStore.getFileStore().getFileOutputStream(getFeedPath()));
            }
        } catch (FeedException fex) {
            throw new AtomException(fex);
        }catch (IOException ex) {
            throw new AtomException(ex);
        }
    }
        
    private InputStream createDefaultFeedDocument(String uri) throws AtomException {
        
        Feed f = new Feed();
        f.setTitle("Feed");
        f.setFeedType( FEED_TYPE);
        
        Link selfLink = new Link();
        selfLink.setRel("self");
        selfLink.setHref(uri);
        f.getOtherLinks().add(selfLink);
                
        try {
            WireFeedOutput wireFeedOutput = new WireFeedOutput();
            Document feedDoc = wireFeedOutput.outputJDom( f );
            XMLOutputter outputter = new XMLOutputter();
            outputter.setFormat(Format.getPrettyFormat());
            outputter.output(feedDoc, 
                FileStore.getFileStore().getFileOutputStream(getFeedPath()));
            
        } catch (FeedException ex) {
            throw new AtomException(ex);
        } catch (IOException ex) {
            throw new AtomException(ex);
        } catch ( Exception e ) {
            
            e.printStackTrace();
        }
        return FileStore.getFileStore().getFileInputStream(getFeedPath());
    }
    
    private Entry loadAtomResourceEntry(InputStream in, File file) {
        try {
            Entry entry = AtomServlet.parseEntry(new BufferedReader(new InputStreamReader(in) ));
            return updateAtomResourceEntry(entry, file);
            
        } catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }    
    }
    
    private Entry updateAtomResourceEntry(Entry entry, File file) {
        
        // Set correct links for entry

        String editURI = appuri + "/" + handle 
                + "/" + singular + "/" + file.getName()  + ".media-link";
        
        String editMediaURI = appuri + "/" + handle 
                + "/" + singular + "/" + file.getName();
        
        String viewURI = appuri + "/" + handle 
                + "/" + collection + "/"+ file.getName() + "/media/" + file.getName();        
        
        // TODO: figure out why PNG is missing from Java MIME types
        FileTypeMap map = FileTypeMap.getDefaultFileTypeMap();
        if (map instanceof MimetypesFileTypeMap) {
            try {
                ((MimetypesFileTypeMap)map).addMimeTypes("image/png png PNG");
            } catch (Exception ignored) {}
        }
        String contentType = map.getContentType(file.getName());
        
        entry.setId(editMediaURI);
        entry.setTitle(file.getName());
        entry.setUpdated(new Date());
        
        List otherlinks = new ArrayList();
        entry.setOtherLinks(otherlinks);
        
        Link editlink = new Link();
        editlink.setRel("edit");
        editlink.setHref(editURI);
        otherlinks.add(editlink);
        
        Link editMedialink = new Link();
        editMedialink.setRel("edit-media");
        editMedialink.setHref(editMediaURI);
        otherlinks.add(editMedialink);
        
        Content content = new Content();
        content.setSrc(viewURI);
        content.setType(contentType);
        List contents = new ArrayList();
        contents.add(content);
        entry.setContents(contents);
                
        return entry;
    }
    
    /**
     * Create a Rome Atom entry based on a Roller entry.
     * Content is escaped.
     * Link is stored as rel=alternate link.
     */    
    private Entry loadAtomEntry(InputStream in)   {
        try {
            return AtomServlet.parseEntry(new BufferedReader(new InputStreamReader(in) ));
            
        } catch ( Exception e ) {
            e.printStackTrace();
            return null;
        }
    }
        
    private Entry updateAtomEntry(Entry entry, String fsid) {
             
        // Set correct links
        String absUrl = appuri + "/" + handle + "/" + singular + "/" + fsid;  
        
        entry.setId(absUrl);

        Link altlink = new Link();
        altlink.setRel("alternate");
        altlink.setHref(absUrl);
        List altlinks = new ArrayList();
        altlinks.add(altlink);
        entry.setAlternateLinks(altlinks);
        
        Link editlink = new Link();
        editlink.setRel("edit");
        editlink.setHref(appuri + "/" + handle + "/" + singular + "/" + fsid);
        List otherlinks = new ArrayList();
        otherlinks.add(editlink);
        entry.setOtherLinks(otherlinks);
        
        entry.setUpdated(new Date());
        return entry;
    }
    
    /**
     * Save file to website's resource directory.
     * @param handle Weblog handle to save to
     * @param name Name of file to save
     * @param size Size of file to be saved
     * @param is Read file from input stream
     */
    private void saveFile(
        String name, String contentType, long size, InputStream is) throws AtomException {
                
        byte[] buffer = new byte[8192];
        int bytesRead = 0;
        
        File dirPath = new File(getEntryMediaPath(name));
        if (!dirPath.exists()) {
            dirPath.mkdirs();
        }
        OutputStream bos = null;
        try {
            bos = new FileOutputStream(
                    dirPath.getAbsolutePath() + File.separator + name);
            while ((bytesRead = is.read(buffer, 0, 8192)) != -1) {
                bos.write(buffer, 0, bytesRead);
            }
        } catch (Exception e) {
            throw new AtomException("ERROR uploading file", e);
        } finally {
            try {
                bos.flush();
                bos.close();
            } catch (Exception ignored) {}
        }
        
    }  
    
    /**
     * Creates a file name for a file based on a weblog handle, title string and a
     * content-type.
     *
     * @param handle      Weblog handle
     * @param title       Title to be used as basis for file name (or null)
     * @param contentType Content type of file (must not be null)
     *
     * If a title is specified, the method will apply the same create-anchor
     * logic we use for weblog entries to create a file name based on the title.
     *
     * If title is null, the base file name will be the weblog handle plus a
     * YYYYMMDDHHSS timestamp.
     *
     * The extension will be formed by using the part of content type that
     * comes after he slash.
     *
     * For example:
     *    weblog.handle = "daveblog"
     *    title         = "Port Antonio"
     *    content-type  = "image/jpg"
     * Would result in port_antonio.jpg
     *
     * Another example:
     *    weblog.handle = "daveblog"
     *    title         = null
     *    content-type  = "image/jpg"
     * Might result in daveblog-200608201034.jpg
     */
    private String createFileName(String title, String contentType) {
        
        if (handle == null) throw new IllegalArgumentException("weblog handle cannot be null");
        if (contentType == null) throw new IllegalArgumentException("contentType cannot be null");
        
        String fileName = null;
        
        SimpleDateFormat sdf = new SimpleDateFormat();
        sdf.applyPattern("yyyyMMddHHssSSS");
        
        // Determine the extension based on the contentType. This is a hack.
        // The info we need to map from contentType to file extension is in
        // JRE/lib/content-type.properties, but Java Activation doesn't provide
        // a way to do a reverse mapping or to get at the data.
        String[] typeTokens = contentType.split("/"); 
        String ext = typeTokens[1];
        
        if (title != null && !title.trim().equals("")) {
            // We've got a title, so use it to build file name
            String base = Utilities.replaceNonAlphanumeric(title, ' ');
            StringTokenizer toker = new StringTokenizer(base);
            String tmp = null;
            int count = 0;
            while (toker.hasMoreTokens() && count < 5) {
                String s = toker.nextToken();
                s = s.toLowerCase();
                tmp = (tmp == null) ? s : tmp + "_" + s;
                count++;
            }
            fileName = tmp + "-" + sdf.format(new Date()) + "." + ext;
            
        } else {
            // No title or text, so instead we'll use the item's date
            // in YYYYMMDD format to form the file name
            fileName = handle + "-" + sdf.format(new Date()) + "." + ext;
        }
        
        return fileName;
    }
        
}
